{*******************************************************************************
 * part of DSPack 1.03                                                                *
 *                                                                             *
 * home page : http://www.progdigy.com                                         *
 * email     : hgourvest@progdigy.com                                          *
 *                                                                             *
 * date      : 27-05-2001                                                      *
 *******************************************************************************}

unit DSUtil;

interface

uses Windows, Messages, Graphics, ActiveX, Sysutils, DirectShow;

const
  WM_GRAPHNOTIFY = WM_APP + 1;      // Filter Graph Event
  IID_IPropertyBag          : TGUID = '{55272A00-42CB-11CE-8135-00AA004BB851}';
  IID_ISpecifyPropertyPages : TGUID = '{B196B28B-BAB4-101A-B69C-00AA00341D07}';
type
  TOverlayColor = record
    case integer of
      0:(low   : cardinal;
         hight : cardinal);
      1:(color : TColor);
    end;

  TFilePath =  array[0..MAX_PATH] of WChar;

// User OPerationS
  TDSUOPSFlag = (
    SelectVideoModePreference,
    PlayTitleOrAtTime,
    PlayChapter,
    PlayTitle,
    Stop,
    ReturnFromSubMenu,
    PlayChapterOrAtTime,
    PlayPrevOrReplayChapter,
    PlayNextChapter,
    PlayForwards,
    PlayBackwards,
    ShowMenuTitle,
    ShowMenuRoot,
    ShowMenuSubPic,
    ShowMenuAudio,
    ShowMenuAngle,
    ShowMenuChapter,
    CanResume,
    SelectOrActivateButton,
    CanStillOff,
    CanPauseOn,
    SelectAudioStream,
    SelectSubPicStream,
    SelectAngle,
    SelectKaraokeAudioPresentationMode

  );

  TDSUOPSKind = set of TDSUOPSFlag;

  TUOPS =  record
    TitlePlay                         : boolean; //2   Finds the media file with the specified title index and plays it back.
    ChapterPlay                       : boolean; //1   Plays the media file with the specified title index and chapter value.
    TimePlay                          : boolean; //0   Plays the media file with the specified title index, starting at the specified time.
    StopForResume                     : boolean; //3   Transitions playback to the DVD_DOMAIN_Stop state after saving resume information.
    GoUp                              : boolean; //4   Halts playback of the current media file and starts playback of the designated previous program chain (PGC).
    TimeSearch                        : boolean; //0,5 Halts playback of the current chapter and starts playback from the specified time in the same media file.
    ChapterSearch                     : boolean; //1,5 Halts playback of the current chapter and starts playback from the specified chapter value in the same media file.
    PrevPGSearch                      : boolean; //6   Halts playback of the current program and starts playback from the previous program within the program chain (PGC).
    TopPGSearch                       : boolean; //6   Halts playback of the current program and restarts playback of the current program within the program chain (PGC).
    NextPGSearch                      : boolean; //7   Halts playback of the current program and starts playback from the next program within the program chain (PGC).
    ForwardScan                       : boolean; //8   Search forward through the current disc at the specified speed.
    BackwardScan                      : boolean; //9   Search backward through the current disc at the specified speed.
    MenuCall_Title_Menu               : boolean; //10  Displays the specified menu on the screen.
    MenuCall_Root_Menu                : boolean; //11
    MenuCall_Sub_picture_Menu         : boolean; //12
    MenuCall_Audio_Menu               : boolean; //13
    MenuCall_Angle_Menu               : boolean; //14
    MenuCall_Chapter_Menu             : boolean; //15
    Resume                            : boolean; //16  Returns to playing back a title from a menu.
    UpperButtonSelect                 : boolean; //17  Selects the upper directional button from the displayed menu.
    LowerButtonSelect                 : boolean; //17  Selects the lower directional button from the displayed menu.
    LeftButtonSelect                  : boolean; //17  Selects the left directional button from the displayed menu.
    RightButtonSelect                 : boolean; //17  Selects the right directional button from the displayed menu.
    ButtonActivate                    : boolean; //17  Activates the selected button.
    ButtonSelectAndActivate           : boolean; //17  Selects and activates the specified button.
    StillOff                          : boolean; //18  Resumes playback, cancelling still mode.
    PauseOn                           : boolean; //19  Pauses the current media file playback.
    PauseOff                          : boolean; //19  Unpauses the current media file playback.
    AudioStreamChange                 : boolean; //20  Sets the current audio stream.
    SubpictureStreamChange            : boolean; //21  Enables or disables picture-in-picture mode and sets the subpicture to the specified source.
    AngleChange                       : boolean; //22  Sets the new display angle.
    KaraokeAudioPresentationModeChange: boolean; //23  Sets the audio playback mode to karaoke.
    VideoModePreferrence              : boolean; //24  Sets the video display mode the user prefers.
  end;

  TDSAMSeeking = (
    CanSeekAbsolute,
    CanSeekForwards,
    CanSeekBackwards,
    CanGetCurrentPos,
    CanGetStopPos,
    CanGetDuration,
    CanPlayBackwards,
    CanDoSegments,
    Source
  );
  TDSAMSeekingKind = set of TDSAMSeeking;

  function FileToPath(FileName: string): TFilePath;
  function GetEventCodeDef(code: longint): string;
  function IntToBin(Value: LongInt; Digits: Integer): String;
  function IntToUOPS(value : longint):TUOPS;
  function IntToTimeCode(x : longint): TDVD_TimeCode;
  function BinToInt(Value: String): LongInt;
  function TimeCodeToFrames(value : TDVD_TimeCode): longint;
  function ShowFilterPropertyPage(Filter: IBaseFilter): HRESULT;
  function AddGraphToRot(Graph: IFilterGraph; out dwRegister: Longint): HRESULT;
  function RemoveGraphFromRot(dwRegister: longint): HRESULT;

implementation

  function  FileToPath(FileName: string): TFilePath;
  begin
    MultiByteToWideChar(CP_ACP,0,Pchar(FileName),-1,result,Sizeof(result) div sizeof(result[0]));
  end;

  function  GetEventCodeDef(code: longint): string;
  begin
    case code of
      EC_ACTIVATE                  : result:= 'EC_ACTIVATE - A video window is being activated or deactivated.';
      EC_BUFFERING_DATA            : result:= 'EC_BUFFERING_DATA - The graph is buffering data, or has stopped buffering data.';
      EC_CLOCK_CHANGED             : result:= 'EC_CLOCK_CHANGED - The reference clock has changed.';
      EC_COMPLETE                  : result:= 'EC_COMPLETE - All data from a particular stream has been rendered.';
      EC_DEVICE_LOST               : result:= 'EC_DEVICE_LOST - A Plug and Play device was removed or has become available again.';
      EC_DISPLAY_CHANGED           : result:= 'EC_DISPLAY_CHANGED - The display mode has changed.';
      EC_END_OF_SEGMENT            : result:= 'EC_END_OF_SEGMENT - The end of a segment has been reached.';
      EC_ERROR_STILLPLAYING        : result:= 'EC_ERROR_STILLPLAYING - An asynchronous command to run the graph has failed.';
      EC_ERRORABORT                : result:= 'EC_ERRORABORT - An operation was aborted because of an error.';
      EC_FULLSCREEN_LOST           : result:= 'EC_FULLSCREEN_LOST - The video renderer is switching out of full-screen mode.';
      EC_GRAPH_CHANGED             : result:= 'EC_GRAPH_CHANGED - The filter graph has changed.';
      EC_NEED_RESTART              : result:= 'EC_NEED_RESTART - A filter is requesting that the graph be restarted.';
      EC_NOTIFY_WINDOW             : result:= 'EC_NOTIFY_WINDOW - Notifies a filter of the video renderer''s window.';
      EC_OLE_EVENT                 : result:= 'EC_OLE_EVENT - A filter is passing a text string to the application.';
      EC_OPENING_FILE              : result:= 'EC_OPENING_FILE - The graph is opening a file, or has finished opening a file.';
      EC_PALETTE_CHANGED           : result:= 'EC_PALETTE_CHANGED - The video palette has changed.';
      EC_PAUSED                    : result:= 'EC_PAUSED - A pause request has completed.';
      EC_QUALITY_CHANGE            : result:= 'EC_QUALITY_CHANGE - The graph is dropping samples, for quality control.';
      EC_REPAINT                   : result:= 'EC_REPAINT - A video renderer requires a repaint.';
      EC_SEGMENT_STARTED           : result:= 'EC_SEGMENT_STARTED - A new segment has started.';
      EC_SHUTTING_DOWN             : result:= 'EC_SHUTTING_DOWN - The filter graph is shutting down, prior to being destroyed.';
      EC_SNDDEV_IN_ERROR           : result:= 'EC_SNDDEV_IN_ERROR - An audio device error has occurred on an input pin.';
      EC_SNDDEV_OUT_ERROR          : result:= 'EC_SNDDEV_OUT_ERROR - An audio device error has occurred on an output pin.';
      EC_STARVATION                : result:= 'EC_STARVATION - A filter is not receiving enough data.';
      EC_STEP_COMPLETE             : result:= 'EC_STEP_COMPLETE - A filter performing frame stepping has stepped the specified number of frames.';
      EC_STREAM_CONTROL_STARTED    : result:= 'EC_STREAM_CONTROL_STARTED - A stream-control start command has taken effect.';
      EC_STREAM_CONTROL_STOPPED    : result:= 'EC_STREAM_CONTROL_STOPPED - A stream-control start command has taken effect.';
      EC_STREAM_ERROR_STILLPLAYING : result:= 'EC_STREAM_ERROR_STILLPLAYING - An error has occurred in a stream. The stream is still playing.';
      EC_STREAM_ERROR_STOPPED      : result:= 'EC_STREAM_ERROR_STOPPED - A stream has stopped because of an error.';
      EC_USERABORT                 : result:= 'EC_USERABORT - The user has terminated playback.';
      EC_VIDEO_SIZE_CHANGED        : result:= 'EC_VIDEO_SIZE_CHANGED - The native video size has changed.';
      EC_WINDOW_DESTROYED          : result:= 'EC_WINDOW_DESTROYED - The video renderer was destroyed or removed from the graph.';
      EC_TIMECODE_AVAILABLE        : result:= 'EC_TIMECODE_AVAILABLE- Sent by filter supporting timecode';
      EC_EXTDEVICE_MODE_CHANGE     : result:= 'EC_EXTDEVICE_MODE_CHANGE - Sent by filter supporting IAMExtDevice';
      EC_CLOCK_UNSET               : result:= 'EC_CLOCK_UNSET - notify the filter graph to unset the current graph clock';
      EC_TIME                      : result:= 'EC_TIME - The requested reference time occurred (currently not used)';

      EC_DVD_ANGLE_CHANGE              : result:= 'EC_DVD_ANGLE_CHANGE - Signals that either the number of available angles changed or that the current angle number changed.';
      EC_DVD_ANGLES_AVAILABLE          : result:= 'EC_DVD_ANGLES_AVAILABLE - Indicates whether an angle block is being played and angle changes can be performed.';
      EC_DVD_AUDIO_STREAM_CHANGE       : result:= 'EC_DVD_AUDIO_STREAM_CHANGE - Signals that the current audio stream number changed for the main title.';
      EC_DVD_BUTTON_AUTO_ACTIVATED     : result:= 'EC_DVD_BUTTON_AUTO_ACTIVATED - Signals that a menu button has been automatically activated per instructions on the disc.';
      EC_DVD_BUTTON_CHANGE             : result:= 'EC_DVD_BUTTON_CHANGE - Signals that either the number of available buttons changed or that the currently selected button number changed.';
      EC_DVD_CHAPTER_AUTOSTOP          : result:= 'EC_DVD_CHAPTER_AUTOSTOP - Indicates that playback stopped as the result of a call to the IDvdControl2::PlayChaptersAutoStop method.';
      EC_DVD_CHAPTER_START             : result:= 'EC_DVD_CHAPTER_START - Signals that the DVD Navigator started playback of a new chapter in the current title.';
      EC_DVD_CMD_START                 : result:= 'EC_DVD_CMD_START - Signals that a particular command has begun.';
      EC_DVD_CMD_END                   : result:= 'EC_DVD_CMD_END - Signals that a particular command has completed.';
      EC_DVD_CURRENT_HMSF_TIME         : result:= 'EC_DVD_CURRENT_HMSF_TIME - Signals the current time in DVD_HMSF_TIMECODE format at the beginning of every VOBU, which occurs every .4 to 1.0 sec.';
      EC_DVD_CURRENT_TIME              : result:= 'EC_DVD_CURRENT_TIME - Signals the beginning of every video object unit (VOBU), a video segment which is 0.4 to 1.0 seconds in length.';
      EC_DVD_DISC_EJECTED              : result:= 'EC_DVD_DISC_EJECTED - Signals that a disc has been ejected from the drive.';
      EC_DVD_DISC_INSERTED             : result:= 'EC_DVD_DISC_INSERTED - Signals that a disc has been inserted into the drive.';
      EC_DVD_DOMAIN_CHANGE             : result:= 'EC_DVD_DOMAIN_CHANGE - Indicates the DVD Navigator''s new domain.';
      EC_DVD_ERROR                     : result:= 'EC_DVD_ERROR - Signals a DVD error condition.';
      EC_DVD_KARAOKE_MODE              : result:= 'EC_DVD_KARAOKE_MODE - Indicates that the Navigator has either begun playing or finished playing karaoke data.';
      EC_DVD_NO_FP_PGC                 : result:= 'EC_DVD_NO_FP_PGC - Indicates that the DVD disc does not have a FP_PGC (First Play Program Chain).';
      EC_DVD_PARENTAL_LEVEL_CHANGE     : result:= 'EC_DVD_PARENTAL_LEVEL_CHANGE - Signals that the parental level of the authored content is about to change.';
      EC_DVD_PLAYBACK_RATE_CHANGE      : result:= 'EC_DVD_PLAYBACK_RATE_CHANGE - Indicates that a playback rate change has been initiated and the new rate is in the parameter.';
      EC_DVD_PLAYBACK_STOPPED          : result:= 'EC_DVD_PLAYBACK_STOPPED - Indicates that playback has been stopped. The DVD Navigator has completed playback of the title and did not find any other branching instruction for subsequent playback.';
      EC_DVD_PLAYPERIOD_AUTOSTOP       : result:= 'EC_DVD_PLAYPERIOD_AUTOSTOP - Indicates that the Navigator has finished playing the segment specified in a call to PlayPeriodInTitleAutoStop.';
      EC_DVD_STILL_OFF                 : result:= 'EC_DVD_STILL_OFF - Signals the end of any still.';
      EC_DVD_STILL_ON                  : result:= 'EC_DVD_STILL_ON - Signals the beginning of any still.';
      EC_DVD_SUBPICTURE_STREAM_CHANGE  : result:= 'EC_DVD_SUBPICTURE_STREAM_CHANGE - Signals that the current subpicture stream number changed for the main title.';
      EC_DVD_TITLE_CHANGE              : result:= 'EC_DVD_TITLE_CHANGE - Indicates when the current title number changes.';
      EC_DVD_VALID_UOPS_CHANGE         : result:= 'EC_DVD_VALID_UOPS_CHANGE - Signals that the available set of IDVDControl2 interface methods has changed.';
      EC_DVD_WARNING                   : result:= 'EC_DVD_WARNING - Signals a DVD warning condition.'
    else
      result := 'Unknow Graph Event!';
    end;
  end;

  function IntToBin(Value: LongInt; Digits: Integer): String;
  var
   i: Integer;
  begin
   Result:='';
   for i:=Digits downto 0 do
    if Value and (1 shl i)<>0 then
     Result:=Result + '1'
    else
     Result:=Result + '0';
  end;

   function IntToUOPS(value : longint):TUOPS;
  var temp : string;
  begin
    temp := inttobin(value,24);
  with result do
  begin
     TitlePlay                          := (copy(temp,3,1)='1'); //2
     ChapterPlay                        := (copy(temp,2,1)='1'); //1
     TimePlay                           := (copy(temp,1,1)='1'); //0
     StopForResume                      := (copy(temp,4,1)='1'); //3
     GoUp                               := (copy(temp,5,1)='1'); //4
     TimeSearch                         := (copy(temp,1,1)='1')and(copy(temp,6,1)='1'); //0,5
     ChapterSearch                      := (copy(temp,2,1)='1')and(copy(temp,6,1)='1'); //1,5
     PrevPGSearch                       := (copy(temp,7,1)='1'); //6
     TopPGSearch                        := (copy(temp,7,1)='1'); //6
     NextPGSearch                       := (copy(temp,8,1)='1'); //7
     ForwardScan                        := (copy(temp,9,1)='1'); //8
     BackwardScan                       := (copy(temp,10,1)='1'); //9
     MenuCall_Title_Menu                := (copy(temp,11,1)='1'); //10
     MenuCall_Root_Menu                 := (copy(temp,12,1)='1'); //11
     MenuCall_Sub_picture_Menu          := (copy(temp,13,1)='1'); //12
     MenuCall_Audio_Menu                := (copy(temp,14,1)='1'); //13
     MenuCall_Angle_Menu                := (copy(temp,15,1)='1'); //14
     MenuCall_Chapter_Menu              := (copy(temp,16,1)='1'); //15
     Resume                             := (copy(temp,17,1)='1'); //16
     UpperButtonSelect                  := (copy(temp,18,1)='1'); //17
     LowerButtonSelect                  := (copy(temp,18,1)='1'); //17
     LeftButtonSelect                   := (copy(temp,18,1)='1'); //17
     RightButtonSelect                  := (copy(temp,18,1)='1'); //17
     ButtonActivate                     := (copy(temp,18,1)='1'); //17
     ButtonSelectAndActivate            := (copy(temp,18,1)='1'); //17
     StillOff                           := (copy(temp,19,1)='1'); //18
     PauseOn                            := (copy(temp,20,1)='1'); //19
     PauseOff                           := (copy(temp,20,1)='1'); //19
     AudioStreamChange                  := (copy(temp,21,1)='1'); //20
     SubpictureStreamChange             := (copy(temp,22,1)='1'); //21
     AngleChange                        := (copy(temp,23,1)='1'); //22
     KaraokeAudioPresentationModeChange := (copy(temp,24,1)='1'); //23
     VideoModePreferrence               := (copy(temp,25,1)='1'); //24
  end;
  end;

  function IntToTimeCode(x : longint): TDVD_TimeCode;
  var
     tc : TDVD_TimeCode;
     str: string;
  begin
    str := IntToBin(x,31);
    tc.Hours1        := bintoint(copy(str,29,4));
    tc.Hours10       := bintoint(copy(str,25,4));
    tc.Minutes1      := bintoint(copy(str,21,4));
    tc.Minutes10     := bintoint(copy(str,17,4));
    tc.Seconds1      := bintoint(copy(str,13,4));
    tc.Seconds10     := bintoint(copy(str,9,4));
    tc.Frames1       := bintoint(copy(str,5,4));
    tc.Frames10      := bintoint(copy(str,3,2));
    tc.FrameRateCode := bintoint(copy(str,1,2));
    result := tc;
end;

  function BinToInt(Value: String): LongInt;
  var
     i,Digits: Integer;
     p : array[0..3]of integer;
  begin
     Result:=0;
     p[0]:=1;p[1]:=2;p[2]:=4;p[3]:=8;
     Digits:=Length(Value);
     for i := 1 to digits do
       begin
         if copy(value,i,1) = '1' then result := result+p[digits-i];
       end;
  end;

  function TimeCodeToFrames(value : TDVD_TimeCode): longint;
  var
    frate : integer;
  begin
    case value.FrameRateCode of
      1 : frate := 25;
      3 : frate := 30;
    else
      frate := 0;
    end;
    result := ( (value.Hours1+value.Hours10*10)*3600 +(value.Minutes1+value.Minutes10*10)*60 +value.Seconds1+value.Seconds10*10)*frate+value.Frames1+value.Frames10*10;
  end;

  function ShowFilterPropertyPage(Filter: IBaseFilter): HRESULT;
  var
    SpecifyPropertyPages : ISpecifyPropertyPages;
    CAGUID  :TCAGUID;
    FilterInfo: TFilterInfo;
  begin
    result := S_FALSE;
    if Filter = nil then exit;
    result := Filter.QueryInterface(IID_ISpecifyPropertyPages, SpecifyPropertyPages);
    if result <> S_OK then exit;
    result := SpecifyPropertyPages.GetPages(CAGUID);
    if result <> S_OK then exit;
    result := Filter.QueryFilterInfo(FilterInfo);
    if result <> S_OK then exit;
    result := OleCreatePropertyFrame(0, 0, 0, FilterInfo.achName, 1, @Filter, CAGUID.cElems, CAGUID.pElems, 0, 0, nil );
  end;

  function AddGraphToRot(Graph: IFilterGraph; out dwRegister: Longint): HRESULT;
  var Moniker: IMoniker;
      ROT    : IRunningObjectTable;
      wsz    : string[255];
  begin
    result := GetRunningObjectTable(0, ROT);
    if (result <> S_OK) then exit;
    wsz := format('FilterGraph %p pid %x',[pointer(graph),GetCurrentProcessId()]);
    result  := CreateItemMoniker('!', StringToOleStr(wsz), Moniker);
    if (result <> S_OK) then exit;
    result  := ROT.Register(0, Graph, Moniker, dwRegister);
  end;

  function RemoveGraphFromRot(dwRegister: longint): HRESULT;
  var ROT: IRunningObjectTable;
  begin
    result := GetRunningObjectTable(0, ROT);
    if (result <> S_OK) then exit;
    result := ROT.Revoke(dwRegister);
  end;

end.
